@@ -19,30 +19,6 @@ def is_success(status_code: int) -> bool:
     return status_code >= 200 and status_code <= 299
 
 
-class Targetdate:
-
-    def __init__(self, start_date: str):
-        self.start_date = start_date
-        self.unit = "seconds"
-        self.max = 86000
-        self.stepsize = 60
-
-    def remaining_secs_to_start(self) -> int:
-        remaining_secs_to_wait = int((datetime.fromisoformat(self.start_date) - datetime.now()).total_seconds())
-        remaining_secs_to_start = int(remaining_secs_to_wait/self.stepsize) * self.stepsize
-        if remaining_secs_to_start < 0:
-            remaining_secs_to_start = 0
-        if remaining_secs_to_start > self.max:
-            remaining_secs_to_start = self.max
-        return remaining_secs_to_start
-
-    def remaining_secs_to_finish(self, program_duration_sec: int) -> int:
-        remaining_secs_to_finish = self.remaining_secs_to_start() + program_duration_sec
-        if remaining_secs_to_finish > self.max:
-            remaining_secs_to_finish = self.max
-        return remaining_secs_to_finish
-
-
 class Appliance(EventListener):
 
     def __init__(self, device_uri: str, auth: Auth, name: str, device_type: str, haid: str, brand: str, vib: str, enumber: str, request_counter: DailyRequestCounter):
@@ -58,10 +34,11 @@ class Appliance(EventListener):
         self.__value_changed_listeners = set()
         self.last_refresh = datetime.now() - timedelta(hours=9)
         self.remote_start_allowed = False
+        self.startable = False
+        self.program_remote_control_active = False
         self._program_selected = ""
         self.program_remaining_time_sec = 0
         self.__program_progress = 0
-        self.__program_remote_control_active = ""
         self.__program_local_control_active = ""
         self._power = ""
         self._door = ""
@@ -131,11 +108,11 @@ class Appliance(EventListener):
         self._on_value_changed_event(event)
 
     def on_status_event(self, event):
-        logging.debug(self.name + " status event: " + str(event.data))
+        #logging.debug(self.name + " status event: " + str(event.data))
         self._on_value_changed_event(event)
 
     def _on_event_event(self, event):
-        logging.debug(self.name + " event event: " + str(event.data))
+        logging.debug(self.name + " unhandled event event: " + str(event.data))
 
     def _on_value_changed_event(self, event):
         try:
@@ -148,10 +125,10 @@ class Appliance(EventListener):
     def _reload_status_and_settings(self):
         self.last_refresh = datetime.now()
         try:
-            settings = self._perform_get('/settings').get('data', {}).get('settings', {})
+            settings = self.__perform_get('/settings').get('data', {}).get('settings', {})
             self._on_values_changed(settings, "reload settings")
 
-            status = self._perform_get('/status').get('data', {}).get('status', {})
+            status = self.__perform_get('/status').get('data', {}).get('status', {})
             self._on_values_changed(status, "reload status")
         except Exception as e:
             self._on_reload_status_and_settings_error(e)
@@ -194,8 +171,8 @@ class Appliance(EventListener):
             self.__program_local_control_active = change.get('value', None)
             logging.info(self.name + " field 'local control active': " + str(self.__program_local_control_active) + " (" + source + ")")
         elif key == 'BSH.Common.Status.RemoteControlActive':
-            self.__program_remote_control_active = change.get('value', False)
-            logging.info(self.name + " field 'remote control active': " + str(self.__program_remote_control_active) + " (" + source + ")")
+            self.program_remote_control_active = change.get('value', False)
+            logging.info(self.name + " field 'remote control active': " + str(self.program_remote_control_active) + " (" + source + ")")
         elif key == 'BSH.Common.Setting.ChildLock': # supported by dishwasher, washer, dryer, ..
             self.child_lock = change.get('value', False)
             logging.info(self.name + " field 'child lock': " + str(self.child_lock) + " (" + source + ")")
@@ -212,7 +189,7 @@ class Appliance(EventListener):
 
     def _reload_selected_program(self):
         # query the selected program
-        selected_data = self._perform_get('/programs/selected').get('data', {})
+        selected_data = self.__perform_get('/programs/selected').get('data', {})
         self._program_selected = selected_data.get('key', "")
         logging.info(self.name + " program selected: " + str(self._program_selected) + " (reload program)")
         selected_options = selected_data.get('options', "")
@@ -221,14 +198,14 @@ class Appliance(EventListener):
         # query available options of the selected program
         if len(self._program_selected) > 0:
             try:
-                available_data = self._perform_get('/programs/available/' + self._program_selected).get('data', {})
+                available_data = self.__perform_get('/programs/available/' + self._program_selected).get('data', {})
                 available_options = available_data.get('options', "")
                 self._on_values_changed(available_options, "reload program")
             except Exception as e:
                 logging.warning("error occurred fetching program options of " + self._program_selected + " " + str(e))
         self._notify_listeners()
 
-    def _perform_get(self, path:str) -> Dict[str, Any]:
+    def __perform_get(self, path:str) -> Dict[str, Any]:
         self.request_counter.inc()
         uri = self._device_uri + path
         response = requests.get(uri, headers={"Authorization": "Bearer " + self._auth.access_token}, timeout=5000)
@@ -249,7 +226,6 @@ class Appliance(EventListener):
                 logging.warning("waiting " + str(delay) + " sec for retry")
                 sleep(delay)
                 self._perform_put(path, data, max_trials, current_trial+1)
-                logging.info(str(current_trial) + ". trial of calling PUT "  + uri + " succeeded")
             else:
                 response.raise_for_status()
 
@@ -310,6 +286,13 @@ class Dishwasher(Appliance):
             return super()._on_value_changed(key, change, source)
         return True
 
+    def _notify_listeners(self):
+        self.startable = self.door.lower() == "closed" and \
+                         self.power.lower() == "on" and \
+                         self.remote_start_allowed and \
+                         self.operation.lower() not in ['delayedstart','run', 'finished', 'inactive']
+        super()._notify_listeners()
+
     def read_start_date(self) -> str:
         start_date = datetime.now() + timedelta(seconds=self.__program_start_in_relative_sec)
         if start_date > datetime.now():
@@ -324,7 +307,7 @@ class Dishwasher(Appliance):
         if len(self._program_selected) == 0:
             logging.warning("ignoring start command. No program selected")
 
-        elif self._operation in ["BSH.Common.EnumType.OperationState.Ready"]:
+        elif self.startable:
             remaining_secs_to_wait = int((datetime.fromisoformat(start_date) - datetime.now()).total_seconds())
             if remaining_secs_to_wait < 0:
                 remaining_secs_to_wait = 0
@@ -342,7 +325,10 @@ class Dishwasher(Appliance):
             }
             try:
                 self._perform_put("/programs/active", json.dumps(data, indent=2), max_trials=3)
-                logging.info(self.name + " program " + self.program_selected + " starts in " + print_duration(remaining_secs_to_wait) + " (program duration " + print_duration(self.program_remaining_time_sec) + ")")
+                logging.info(self.name + " PROGRAMSTRART - program " + self.program_selected +
+                             " starts in " + print_duration(remaining_secs_to_wait) +
+                             " (duration " + print_duration(self.program_remaining_time_sec) + ")")
+
             except Exception as e:
                 logging.warning("error occurred by starting " + self.name + " " + str(e))
         else:
@@ -396,10 +382,13 @@ class Dryer(Appliance):
                 logging.info(self.name + " field 'finish in relative': " + str(self.__program_finish_in_relative_sec) + " (" + source + ")")
             if 'constraints' in change.keys():
                 constraints = change['constraints']
+                print(constraints)
                 if 'max' in constraints.keys():
                     self.__program_finish_in_relative_max_sec = constraints['max']
+                    logging.info(self.name + " field 'program_finish_in_relative_max_sec: " + str(self.__program_finish_in_relative_max_sec) + " (" + source + ")")
                 if 'stepsize' in constraints.keys():
                     self.__program_finish_in_relative_stepsize_sec = constraints['stepsize']
+                    logging.info(self.name + " field 'program_finish_in_relative_stepsize_sec: " + str(self.__program_finish_in_relative_stepsize_sec) + " (" + source + ")")
         elif key == 'LaundryCare.Dryer.Option.DryingTarget':
             self.__program_drying_target = change.get('value', "")
         elif key == 'LaundryCare.Dryer.Option.DryingTargetAdjustment':
@@ -413,6 +402,14 @@ class Dryer(Appliance):
             return super()._on_value_changed(key, change, source)
         return True
 
+    def _notify_listeners(self):
+        self.startable = self.door.lower() == "closed" and \
+                         self.power.lower() == "on" and \
+                         self.remote_start_allowed and \
+                         self.program_remote_control_active and \
+                         self.operation.lower() not in ['delayedstart','run', 'finished', 'inactive']
+        super()._notify_listeners()
+
     def __compute_program_duration(self):
         duration = self.__program_finish_in_relative_sec
         if duration == 0:
@@ -445,14 +442,20 @@ class Dryer(Appliance):
         if len(self._program_selected) == 0:
             logging.warning("ignoring start command. No program selected")
 
-        elif self._operation in ["BSH.Common.EnumType.OperationState.Ready", ''] and self.power.upper() == 'ON':
+        elif self.startable:
             self.__program_duration_sec = self.__compute_program_duration()
             remaining_secs_to_finish = int((datetime.fromisoformat(end_date) - datetime.now()).total_seconds())
+            logging.info("remaining_secs_to_finish " + print_duration(remaining_secs_to_finish) + " computed based on end date " + end_date)
             if remaining_secs_to_finish < 0:
+                logging.info("remaining_secs_to_finish is < 0. set it with 0")
                 remaining_secs_to_finish = 0
             if remaining_secs_to_finish > self.__program_finish_in_relative_max_sec:
+                logging.info("remaining_secs_to_finish " + print_duration(remaining_secs_to_finish) + " > " + print_duration(self.__program_finish_in_relative_max_sec) + " max allowed. set it with " + print_duration(self.__program_finish_in_relative_max_sec))
                 remaining_secs_to_finish = self.__program_finish_in_relative_max_sec
-            remaining_secs_to_finish = int(remaining_secs_to_finish / self.__program_finish_in_relative_stepsize_sec) * self.__program_finish_in_relative_stepsize_sec
+            stepsize = self.__program_finish_in_relative_stepsize_sec
+            if stepsize > 0:
+                remaining_secs_to_finish = int(remaining_secs_to_finish / stepsize) * stepsize
+                logging.info("remaining_secs_to_finish " + print_duration(remaining_secs_to_finish) + " computed beed onn stepsize of " + str(stepsize) + " sec")
             data = {
                 "data": {
                     "key": self._program_selected,
@@ -465,7 +468,7 @@ class Dryer(Appliance):
             }
             try:
                 self._perform_put("/programs/active", json.dumps(data, indent=2), max_trials=3)
-                logging.info(self.name + " program " + "self.program_selected" + " starts in " + print_duration(remaining_secs_to_finish - self.__compute_program_duration()))
+                logging.info(self.name + " program " + self.program_selected + " starts in " + print_duration(remaining_secs_to_finish - self.__compute_program_duration()))
             except Exception as e:
                 logging.warning("error occurred by starting " + self.name + " " + str(e))
         else:
